#include <ros/ros.h>
#include <nav_msgs/Odometry.h>
#include <geometry_msgs/PoseStamped.h>
#include <geometry_msgs/Twist.h>
#include <geometry_msgs/Vector3.h>
#include <math.h>
//#include <tf2/LinearMath/Quaternion.h>
#include <tf/transform_broadcaster.h>

# define M_PI           3.14159265358979323846  /* pi */

float goalX = 1.0f, goalY = 1.0f, goalAngleZ = 0.0f, goalAngleW = 0.0f;
float odomX = 0.0f, odomY = 0.0f, odomAngle = 0.0f;

void poseMessageReceived(const nav_msgs::Odometry& odm){
    odomX = odm.pose.pose.position.x;
    odomY = odm.pose.pose.position.y;
    
    tf::Pose pose;
    tf::poseMsgToTF(odm.pose.pose, pose);
    odomAngle = tf::getYaw(pose.getRotation());

    //ROS_INFO("x: %.3f y: %.3f", odomX, odomY);
    // x = distance*cos(angle)
    // y = distance*sin(angle)

    // cos(angle) = x/distance
    // angle = cos-1(x/distance)
    
}

void goalMessageReceived(const geometry_msgs::PoseStamped& goal){
    goalX = goal.pose.position.x;
    goalY = goal.pose.position.y;
    ROS_INFO("x: %f y: %f ", goalX, goalY);
    
}

double GetRadianDegree(float originx, float originy, float targetx, float targety) {
    double lol = std::atan2(originy - targety, originx - targetx) + M_PI;
    if(lol > M_PI) {
        lol = 2*M_PI - lol;
    }
    return lol;
}


int main(int argc, char **argv){
    ros::init(argc, argv, "moj_paket");

    ros::NodeHandle nd;

    
    ros::Subscriber odomSub = nd.subscribe("/odom", 1000, &poseMessageReceived);
    ros::Subscriber goalSub = nd.subscribe("/move_base_simple/goal", 1000, &goalMessageReceived);

    ros::Publisher pub = nd.advertise<geometry_msgs::Twist>( "cmd_vel" , 1 );

    ros::Rate loopRate(1);

    while(ros::ok()){

        
        //float yaw = tf2::getYaw(odm.pose.pose.orientation);
        

        

        float distance = std::sqrt( std::pow((goalX - odomX), 2) + std::pow((goalY - odomY), 2) );
        
        geometry_msgs::Twist go_to;
        geometry_msgs::Vector3 linear, angular;
        linear.x = 0.0f;
        linear.y = 0.0f;
        linear.z = 0.0f;
        angular.x = 0.0f;
        angular.y = 0.0f;
        angular.z = 0.0f;

        if(distance > 0.3f){

            

            //float robotVectorX = cos(odomAngle);
            //float robotVectorY = sin(odomAngle);

            //float desiredAngle = std::acos(goalX/distance);
            //GetAngleDegree(0, 0, goalX-odomX, goalY-odomY)

            float zeljeniKut = std::atan(std::abs(odomY - goalY)/std::abs(odomX - goalX));
            if(goalX < odomX && goalY > odomY) {
                zeljeniKut = M_PI - zeljeniKut;
            } else if(goalX > odomX && goalY < odomY) {
                zeljeniKut = -zeljeniKut;
            } else if(goalX < odomX && goalY < odomY){
                zeljeniKut = -M_PI + zeljeniKut;
            }
            //if()
            float razlikaKuteva = std::abs(zeljeniKut - odomAngle);
            //float razlikaKuteva = std::abs(GetRadianDegree(odomX, odomY, goalX, goalY) - odomAngle);//acos((robotVectorX*(odomX-goalX) + robotVectorY*(odomY-goalY))/(std::sqrt(robotVectorX*robotVectorX + robotVectorY*robotVectorY)*std::sqrt((odomX-goalX)*(odomX-goalX) + (odomY-goalY)*(odomY-goalY))));
            ROS_INFO("diff: %.2f odomangle: %.2f GRD: %.2f", razlikaKuteva, odomAngle, zeljeniKut);
            //ROS_INFO("robox: %.2f roboy: %.2f", robotVectorX, robotVectorY);
            //angular.z = 0.2f;
            //if( razlikaKuteva < 0.35f ) ROS_INFO("da");

            
            if( razlikaKuteva > 0.1f ){
                // usmjeri robota
                float predznak = 1.f;
                if(zeljeniKut > odomAngle){
                    if(zeljeniKut - odomAngle < M_PI/2.){
                        predznak = 1.f;
                    } else {
                        predznak = -1.f;
                    }
                } else {
                    if(zeljeniKut - odomAngle < M_PI/2.){
                        predznak = -1.f;
                    } else {
                        predznak = 1.f;
                    }
                }

                float Kp = 0.15f*predznak;
                angular.z = Kp*razlikaKuteva;
                if(angular.z > 1.0f) angular.z = 1.0f;
            } else {
                angular.z = 0.f;
            }
            if( razlikaKuteva < 0.5f ){
                if(distance > 0.1f){
                    linear.x = distance*0.2f;
                    if(linear.x > 1.0f) linear.x = 1.0f;
                } else {
                    linear.x = 0.0f;
                }
            }
            
            
        } else {
            angular.z = 0.0f;    
        }

        go_to.linear = linear;
        go_to.angular = angular;

        pub.publish(go_to);


        ros::spinOnce();
        loopRate.sleep();
    }
    

    


    return 0;
}

